package Rel::UXRT731::Sol11sparc;
@Rel::UXRT731::Sol11sparc::ISA = qw(Rel::UXRT731::SunOS);

sub basic_precheck_sys {
    my ($rel,$sys)=@_;
    my ($prod,$msg,$ivers,$cprod,@localzones);

    $cprod=CPIC::get('prod');
    $prod=$sys->prod($cprod);
    $ivers=$prod->version_sys($sys);
    # fix e2009315: block cross stack upgrade for SF basic installer
    if ((Cfg::opt('prodmode') eq 'SF Basic') && ($prod->{prod} ne 'SF') && $ivers && Cfg::opt(qw(upgrade patchupgrade hotfixupgrade))) {
        $msg=Msg::new("$prod->{abbr} version $ivers cannot be upgraded to $prod->{abbr} version $prod->{vers} on $sys->{sys} using SF basic installer");
        $sys->push_error($msg);
        return 0;
    }
    #########################################################
    # HANDLING LOCAL ZONE
    # Check and record all the other zones having ODM running
    #########################################################
    my $zones = $sys->cmd("_cmd_zoneadm list 2>/dev/null");
    my @zonelist = split("\n",$zones);
    $sys->set_value('odmenabledlocalzones','list', @localzones);
    if ( scalar(@zonelist) > 1) { # If zoneadm list gives more than 1 zone means local zones present
       foreach my $zonename (@zonelist) {
           check_odm_localzone_sys($sys,$zonename) if ($zonename ne "global");
       }
    }
    return 1;
}

sub check_odm_localzone_sys {
    my ($sys,$lzname) = @_;
    my ($rtn);

    my $rtn = $sys->cmd("zlogin $lzname _cmd_pkg info VRTSodm 2>/dev/null | /usr/bin/awk '/Name:/ {print \$2}' ");
    chomp($rtn);
    if ($rtn eq 'VRTSodm') {
        $sys->set_value('odmenabledlocalzones','push', "$lzname");
    }
}

package Proc::odm731::SunOS;
@Proc::odm731::SunOS::ISA = qw(Proc::odm731::Common);

sub init_plat {
    my ($proc,$sys)=@_;
    $proc->{controlfile}='/etc/rc2.d/S92odm';
    $proc->{smf_manifest}='/var/svc/manifest/system/vxodm/odm.xml';
    return;
}

sub check_sys {
    my ($proc,$sys,$state)=@_;
    my ($mod,$prod,$rtn,$vxodmd,$cpic,$rel,$activecomponent);
    $cpic = Obj::cpic();
    $rel = $cpic->rel;
    $activecomponent = $rel->get_activecomponent();
    $prod = $proc->prod($activecomponent);
    $mod=$sys->get_drivers($proc->{proc});
    if ($mod eq '') {
        Msg::log("odm not loaded on sys $sys->{sys}");
        return 0;
    } else {
        Msg::log("odm loaded on sys $sys->{sys}");
    }
    if (($state =~ /start/m) && ($prod->{prod} =~ /^(SFRAC|SVS|SFCFS|SFCFSHA)$/mx)) {
        $vxodmd=$sys->cmd('_cmd_gabconfig -a 2>/dev/null');
        return ($vxodmd !~ /Port d/m) ? 0 : 1;
    }
    $rtn=$sys->cmd("_cmd_mount 2> /dev/null | _cmd_grep -w '/dev/odm'");
    if ($rtn ne '') {
        return 1;
    } else {
        Msg::log("odm loaded not configured on $sys->{sys}");
        # stopping, need to unload
        return 1 if ($state =~ /stop/m);
    }
    # starting, need to configure
    return 0;
}

sub start_sys {
    my ($proc,$sys)=@_;
    if ($sys->exists($proc->{smf_manifest})) {
        $sys->cmd("_cmd_svcadm disable -s system/vxodm 2>/dev/null");
        $sys->cmd("_cmd_svcadm enable system/vxodm 2>/dev/null");
    } elsif ($sys->exists($proc->{controlfile})) {
        $sys->cmd("$proc->{controlfile} restart 2>/dev/null");
    } else {
        return 0;
    }
    start_localzone_sys($proc,$sys);
    return 1;
}

sub start_localzone_sys {
    my ($proc,$sys) = @_;
    foreach my $lzname (@{$sys->{odmenabledlocalzones}}) {
        if ($sys->exists($proc->{smf_manifest})) {
            $sys->cmd("zlogin $lzname _cmd_svcadm disable -s system/vxodm 2>/dev/null");
            $sys->cmd("zlogin $lzname _cmd_svcadm enable system/vxodm 2>/dev/null");
        } elsif ($sys->exists($proc->{controlfile})) {
            $sys->cmd("zlogin $lzname $proc->{controlfile} restart 2>/dev/null");
        } else {
            return 0;
        }
    }
}

sub stop_sys {
    my ($proc,$sys)=@_;
    stop_localzone_sys($proc,$sys);
    if ($sys->exists($proc->{smf_manifest})) {
        $sys->cmd("_cmd_svcadm disable -st system/vxodm 2>/dev/null");
    } elsif ($sys->exists($proc->{controlfile})) {
        $sys->cmd("$proc->{controlfile} stop 2>/dev/null");
    }
    $sys->unload_driver($proc->{proc});
    return 1;
}

sub stop_localzone_sys {
    my ($proc,$sys)=@_;
    foreach my $lzname (@{$sys->{odmenabledlocalzones}}) {
        if ($sys->exists($proc->{smf_manifest})) {
            $sys->cmd("zlogin $lzname _cmd_svcadm disable -st system/vxodm 2>/dev/null");
        } elsif ($sys->exists($proc->{controlfile})) {
            $sys->cmd("zlogin $lzname $proc->{controlfile} stop 2>/dev/null");
        }
    }

}

package Padv::Sol11sparc;
@Padv::Sol11sparc::ISA = qw(Padv::SunOS);

sub pkgs_uninstall_sys {
    my ($padv,$sys,$pkgs)=@_;
    my ($rootpath,$uof,$ret);

    $rootpath=Cfg::opt('rootpath')||'';
    $rootpath="-R $rootpath" if ($rootpath);

    my (@updatepkgs, $updatepkgs);
    push (@updatepkgs, 'VRTSodm') if ($pkgs =~ /VRTSodm/);
    push (@updatepkgs, 'VRTSvxfs') if ($pkgs =~ /VRTSvxfs/);
    $updatepkgs = join (' ',@updatepkgs);
    if ( @{$sys->{odmenabledlocalzones}} ) {
        $uof=EDRu::outputfile('uninstall_zone', $sys->{sys}, 'pkgs');
        foreach my $lzname ( @{$sys->{odmenabledlocalzones}} ) {
            if ( @updatepkgs && Cfg::opt(qw(upgrade patchupgrade hotfixupgrade)) ) {
                $ret=$sys->cmd("zlogin $lzname _cmd_pkg $rootpath uninstall --no-backup-be $updatepkgs 2>$uof 1>&2");
                ###cleanup odm hold on zone
                $ret = $sys->cmd("zonecfg -z $lzname remove device match=/dev/odm 2>/dev/null");
                $ret = $sys->cmd("zoneadm -z $lzname apply 2>/dev/null");
            }
            if ( Cfg::opt('uninstall') ) {
                $sys->cmd("zonecfg -z $lzname remove fs dir=/etc/vx/licenses/lic 2>/dev/null");
                $sys->cmd("zoneadm -z $lzname apply 2>/dev/null");
            }
        }
    }

    $uof=EDRu::outputfile('uninstall', $sys->{sys}, 'pkgs');
    $ret=$sys->cmd("_cmd_pkg $rootpath uninstall --no-backup-be $pkgs 2>$uof 1>&2");

    return $ret;
}

sub pkgs_install_sys {
    my ($padv,$sys,$archive,$pkgs)=@_;
    my ($rootpath,$iof,$path,$stat);

    $rootpath=Cfg::opt('rootpath')||'';
    $rootpath="-R $rootpath" if ($rootpath);
    $iof=EDRu::outputfile('install', $sys->{sys}, 'pkgs');
    $path=EDRu::dirname($archive);
    $stat=$sys->filestat($path);
    $sys->cmd("_cmd_chmod a+x $path");

    #$sys->cmd("_cmd_pkg $rootpath install -g $archive $pkgs 2>$iof 1>&2");
    # 'pkg install -g VRTSpkgs.p5p <pkgs>' do not work well when some zones in running state
    # According to 2707052: call set-publisher first
    $sys->cmd("_cmd_pkg $rootpath set-publisher -p $archive Veritas 2>$iof 1>&2; _cmd_pkg $rootpath install --accept --no-backup-be $pkgs 2>>$iof 1>&2; _cmd_pkg $rootpath unset-publisher Veritas 2>>$iof 1>&2; _cmd_svcadm clear application/pkg/system-repository 2>/dev/null");

    my (@updatepkgs, $updatepkgs);
    push (@updatepkgs, 'VRTSodm') if ($pkgs =~ /VRTSodm/);
    push (@updatepkgs, 'VRTSvxfs') if ($pkgs =~ /VRTSvxfs/);
    $updatepkgs = join (' ',@updatepkgs);
    if ( @updatepkgs ) {
        if ( @{$sys->{odmenabledlocalzones}} ) {
            if ( Cfg::opt(qw(upgrade patchupgrade hotfixupgrade)) ) {
                $iof=EDRu::outputfile('install_zone', $sys->{sys}, 'pkgs');
                foreach my $lzname ( @{$sys->{odmenabledlocalzones}} ) {
                    # Add device /dev/odm before installing
                    $sys->cmd("zonecfg -z $lzname \"add device; set match=/dev/odm; end\" 2>/dev/null");
                    $sys->cmd("zoneadm -z $lzname apply 2>/dev/null");
                    $sys->cmd("_cmd_pkg $rootpath set-publisher -p $archive Veritas 2>$iof 1>&2; zlogin $lzname _cmd_pkg $rootpath install --accept --no-backup-be $updatepkgs 2>>$iof 1>&2; _cmd_pkg $rootpath unset-publisher Veritas 2>>$iof 1>&2; _cmd_svcadm clear application/pkg/system-repository 2>/dev/null");
                }
            }
        }
    }

    $sys->change_filestat($path,$stat);
    return 1;
}
package Rel::UXRT731::Common;
@Rel::UXRT731::Common::ISA = qw(Rel);

#e3945461 -- CPI_7.3.1_P1.pl:line 264 -- Upgrade from 7.3 to 7.3.1.100 installs only 7.3.1.100 patch packages
sub check_mixed_product_cross_systems {
    my ($rel) = @_;
    my ($cpic,$cfg,$installedprod,$installedprodvers,$error,$msg,$spm,$cfgprod,$targetprod);
    $cpic=Obj::cpic();
    $cfg=Obj::cfg();

    $installedprod=${$cpic->{systems}}[0]->{upgradeprod}[0];
    $installedprodvers=${$cpic->{systems}}[0]->{prodvers}[0];
    #Error out If the product specified in responsefile is not consistent with the product installed during configuration.
    $cfgprod = $cfg->{prod} || '';
    $cfgprod = uc($cfgprod);
    if (Cfg::opt('configure') && Cfg::opt('responsefile') && $cfgprod ne $installedprod) {
        $installedprod =~ s/\d+//;
        $cfgprod =~ s/\d+//;
        if(Cfg::opt('vomplugin')) {
            $msg=Msg::new("The product $cfgprod selected on the web page is not consistent with the installed product $installedprod.");
        } else {
            $msg=Msg::new("The product $cfgprod specified in the responsefile is not consistent with the installed product $installedprod.");
        }
        ${$cpic->{systems}}[0]->push_error($msg);
        $rel->set_value_allsys('stop_checks',1);
        return 1;
    }
    for my $sys (@{$cpic->{systems}}) {
        if ($installedprod && $sys->{upgradeprod}[0] && ($installedprod ne $sys->{upgradeprod}[0])) {
            $error=1;
            last;
        }
    }
    if ($error) {
        $spm=$rel->system_product_map();
        $msg=Msg::new("Entered systems have different products installed:\n$spm\nSystems running different products must be operated independently");
        ${$cpic->{systems}}[0]->push_error($msg);
        $rel->set_value_allsys('stop_checks',1);
        return 1;
    }

    if (Cfg::opt('precheck')) {
        if (EDRu::compvers($installedprodvers,$rel->{vers},2)==2) {
            Cfg::set_opt('upgrade');
        }
    }

    if (Cfg::opt(qw/install upgrade/)) {
        if (EDRu::compvers($installedprodvers,$rel->{vers},2)==2) {
            if ($rel->{padv} =~ /^Sol1[12]x64/) {
                $rel->check_supported_coexistence_mix_product();
            } else {
                $cpic->set_task('upgrade') if(Cfg::opt('install'));
            }
        } elsif ((EDRu::compvers($installedprodvers,$rel->{vers},3)==0) && ($rel->{type} =~ /MH/)) {
                # base + MR already installed
                $cpic->set_task('hotfixupgrade');
        } elsif (EDRu::compvers($installedprodvers,$rel->{vers},3)==0 && !Cfg::opt('upgrade_nonkernelpkgs')) {
            # base already installed
            if ($rel->{type} eq 'BH') {
                $cpic->set_task('hotfixupgrade');
            } elsif ($rel->{type} eq 'BMH') {
                $cpic->set_task('patchupgrade');
            }
        }
    }
    # If it's Upgrade and $cpic->{prod} is set and isn't $rel->{upgrade_matrix}{$cpic->{systems}[0]->{upgradeprod}[0]}
    # It means the task is changed from Install to Upgrade, and the installed product is not the product to be upgraded.
    if (Cfg::opt('upgrade') && $cpic->{prod} && ($cpic->{prod} ne $rel->{upgrade_matrix}{$installedprod})) {
        $installedprod =~ s/\d+//;
        $targetprod = $cpic->prod->{abbr};
        $targetprod =~ s/\d+//;
        $msg=Msg::new("$installedprod is installed on all the systems. A direct upgrade from $installedprod to $targetprod is not supported.");
        ${$cpic->{systems}}[0]->push_error($msg);
        $rel->set_value_allsys('stop_checks',1);
        return 1;
    }

    # $cpic->{prod} must be set by now
    if (!$cpic->{prod} || Cfg::opt(qw/upgrade patchupgrade hotfixupgrade rollback start stop license uninstall postcheck/)) {
        $cpic->set_value('prod',$rel->get_upgrade_matrix_product(${$cpic->{systems}}[0]->{upgradeprod}[0]));
        $cfg->set_value('prod',$cpic->{prod});
    }
    return 0;
}

package CPIC;
use strict;

#e3948728  -- CPI_7.3.1_P3.pl:line 360-361 -- InfoScale 7.3.1 installation fails if the CPI installer auto downloads the applicable OS update patches that are available on SORT.
sub apply_must_have_patch {
    my ($cpic,$patch_path) = @_;
    my ($msg,$pm,$install_script,$install_path,$cmd);
    my $edr=Obj::edr();
    my $rel=Obj::rel();

    $install_script= $cpic->{script};
    $install_path = $cpic->{scriptdir};
    if ($patch_path && ! -d $patch_path) {
        return 0;
    } else {
        if (!-d "$patch_path/scripts") {
            $msg = Msg::new("The downloaded patch is invalid. Contact our technical support for further troubleshooting.");
            $msg->error;
            return 0;
        }
    }

    # Add this hotfix
    my ($path1,$path2,$sub);
    for my $path (qw/patch_path patch2_path patch3_path patch4_path patch5_path/) {
        if (Cfg::opt("$path")) {
            $path2=$path;
            next;
        }
        $path1=$path;
        last;
    }

    Cfg::set_opt($path1,$patch_path);
    $sub="process_$path1\_arg";
    $sub=~s/patch/hotfix/;
    $cpic->$sub($path1,$path2);

    # create hotfix and patch object
    my ($hf,$hotfix,$pkg,$patch,$pkglist,@pkgs,$prod,$subprod,$padv);
    $padv=$rel->{padv};
    $hf=$cpic->{hotfixes}[-1];
    $hotfix=init_cpip_object("Hotfix::$hf",$padv);
    push(@{$cpic->{hotfixobj}},$hotfix);

    $hotfix->{arg}=$cpic->{hotfix_arg}{$hf} if (defined($cpic->{hotfix_arg}{$hf}));
    $hotfix->define_patches();

    # re-initiate $pkg objects since they may be affected by the new patches
    push(@pkgs, @{$rel->allpkgs});
    for my $prodv (@{$rel->{prods}}) {
        $prod=$rel->prod($prodv);
        push(@pkgs, @{$prod->allpkgs_on_media});
        for my $subprodv(@{$prod->{subprods}}) {
            $prod=$rel->prod($subprodv);
            push(@pkgs, @{$prod->allpkgs_on_media});
        }
    }

    $pkglist=EDRu::arruniq(@pkgs);
    for my $pkgi (@{$pkglist}) {
        $pkg=init_cpip_object("Pkg::$pkgi",$padv);
        $pkg->init();
    }

    for my $patchi (@{$hotfix->{patches}}) {
        $patch=new_object("Patch::$patchi",$padv);
        if ($patch->{patch} =~ /^([a-zA-Z0-9]+)_(\d+)_(\d+)_(\d+)/mxg) {
            $patch->{pkg}="$1$2$3$4";
        }
        $patch->{arg}=$rel->{patcharg}{$patchi} if (defined($rel->{patcharg}{$patchi}));
    }
    # re-adjust package list
    $rel->adjust_pkglist();
    # re-verify media
    $cpic->verify_media($cpic->rel);
    return 1;
}

# Return 1, continue shutdown processes.
# Return 0, skip whole shutdown phase.
sub preshutdown {
    my ($cpic,$prod)=@_;
    my ($steps,$edr,$rel);

    $rel = $cpic->rel();
    # If -makeresponsefile but no prod specified, skip whole shutdown phase.
    return 0 if (Cfg::opt('makeresponsefile') && !($prod));
    # If localization product, skip whole shutdown phase.
    return 0 if ($prod->{lp});

    # If do not re-configure product, skip whole shutdown phase.
    return 0 if ($prod->donotreconfig());

    if (Cfg::opt('rootpath')) {
        # need run prestop for Live upgrade case.
        $steps=1;
    } else {
        $cpic->{stopprocs}=$prod->stopprocs();
        for my $sys (@{$cpic->{systems}}) {
            $sys->{stopprocs}=$prod->stopprocs_sys($sys);
        }
        $cpic->{stopprocs}=$cpic->verify_list('stopprocs');
        # Handling veki unload for 'installer -stop' operation
        if( Cfg::opt('stop') && $prod->{prod} !~ /^VCS$/) {
            $rel->adjust_procs_list($cpic);
        }
        $steps=scalar(@{$cpic->{stopprocs}});
    }

    # If no shutdown tasks, skip whole shutdown phase.
    return 0 if (!$steps);

    $cpic->{falsefailures}=0;

    $prod->pre_shutdown_question();

    if (!Cfg::opt('rootpath')) {
        $edr=Obj::edr();
        if(Cfg::opt('upgrade_kernelpkgs')) {
            $cpic->rel->ru_switch_groups($cpic->{systems});
        }
        $edr->exit_noexitfile() if ($cpic->cli_ask_stop_processes($prod));
    }

    # Show stop processes UI form
    $cpic->show_ui_form('stopprocess');

    return 1;
}

#e3949312 -- CPI_7.3.1_P4.pl:line 382-388 -- GAB service removed from /etc/name_to_major using rem_drv. Adding it back using add_drv
package Padv::Sol11sparc;
@Padv::Sol11sparc::ISA = qw(Padv::SunOS);

sub load_driver_sys {
    my ($padv,$sys,$driver,$donot_check)=@_;
    my ($mn,$drv_added);
    $mn=$padv->driver_sys($sys, $driver) if (!$donot_check);
    if (!$mn) {
        $sys->cmd("_cmd_adddrv $driver 2>/dev/null; _cmd_modload -p drv/$driver 2>/dev/null");
    } else {
        $drv_added=$sys->cmd("_cmd_grep -w $driver /etc/name_to_major 2>/dev/null");
        if (!$drv_added) {
            Msg::log("The module $driver is loaded, but doesn't have an entry in /etc/name_to_major. Adding it now");
            $sys->cmd("_cmd_adddrv $driver 2>/dev/null");
        }
    }
    return '';
}

#e3950736 -- CPI_7.3.1_P5.pl:line 393-605 LLT Over TCP is not configured correctly
package Prod::VCS731::Common;
@Prod::VCS731::Common::ISA = qw(Prod);
sub display_config_info {
    my ($prod,$cname,$cid,$rhbn) = @_;
    my ($address,$cfg,$lmsg,$msg,$netmask,$key,$port,$smsg,$sys,$addrkey,$maskkey,$portkey);
    $cfg = Obj::cfg();

    $smsg = '';
    Msg::title();
    $msg = Msg::new("Cluster information verification:\n");
    $msg->bold;
    $msg=Msg::new("\tCluster Name: \t   $cname");
    $msg->print;
    $smsg .= "$msg->{msg}\n";
    $msg=Msg::new("\tCluster ID Number: $cid");
    $msg->print;
    $smsg .= "$msg->{msg}\n";
    for my $sysi (@{CPIC::get('systems')}) {
        $sys=$sysi->{sys};
        $lmsg='';
        for my $n (1..$prod->num_hbnics($rhbn)) {
            $key="lltlink$n";
            next unless ($rhbn->{$key}{$sys});
            if ($cfg->{lltoverudp}) {
                $address=$rhbn->{"udplink$n".'_address'}{$sys};
                $netmask=$rhbn->{"udplink$n".'_netmask'}{$sys};
                $port=$rhbn->{"udplink$n".'_port'}{$sys};
                if (EDRu::ip_is_ipv6($address)) {
                    $msg=Msg::new("\n\t\tlink$n=$rhbn->{$key}{$sys} over UDP6\n\t\t  ip $address prefix $netmask port $port");
                } else {
                    $msg=Msg::new("\n\t\tlink$n=$rhbn->{$key}{$sys} over UDP\n\t\t  ip $address netmask $netmask port $port");
                }
                $lmsg.=$msg->{msg};
            } elsif ($cfg->{lltovertcp}) {
                $address=$rhbn->{"tcplink$n".'_address'}{$sys};
                $netmask=$rhbn->{"tcplink$n".'_netmask'}{$sys};
                $port=$rhbn->{"tcplink$n".'_port'}{$sys};
                if (EDRu::ip_is_ipv6($address)) {
                    $msg=Msg::new("\n\t\tlink$n=$rhbn->{$key}{$sys} over TCP6\n\t\t  ip $address prefix $netmask port $port");
                } else {
                    $msg=Msg::new("\n\t\tlink$n=$rhbn->{$key}{$sys} ove TCP\n\t\t  ip $address netmask $netmask port $port");
                }
                $lmsg.=$msg->{msg};
            } else {
                $addrkey = "rdmalink${n}_address";
                $maskkey = "rdmalink${n}_netmask";
                $portkey = "rdmalink${n}_port";
                if (defined $rhbn->{$addrkey}) {
                    $address = $rhbn->{$addrkey}{$sys};
                    $netmask = $rhbn->{$maskkey}{$sys};
                    $port = $rhbn->{$portkey}{$sys};
                    $msg=Msg::new("\n\t\tlink$n=$rhbn->{$key}{$sys} over RDMA\n\t\t  ip $address netmask $netmask port $port");
                    $lmsg.=$msg->{msg};
                } else {
                    $msg=Msg::new("\n\t\tlink$n=$rhbn->{$key}{$sys}");
                    $lmsg.=$msg->{msg};
                }
            }
        }
        Msg::n();
        $msg=Msg::new("\tPrivate Heartbeat NICs for $sys: $lmsg");
        $msg->print;
        $smsg .= "$msg->{msg}\n";
        if ($prod->num_lopri_hbnics($rhbn) > 0) {
            $lmsg = '';
            for my $n (1..$prod->num_lopri_hbnics($rhbn)) {
                $key="lltlinklowpri$n";
                if ($cfg->{lltoverudp}) {
                    $address=$rhbn->{"udplinklowpri$n".'_address'}{$sys};
                    $netmask=$rhbn->{"udplinklowpri$n".'_netmask'}{$sys};
                    $port=$rhbn->{"udplinklowpri$n".'_port'}{$sys};
                    if (EDRu::ip_is_ipv6($address)) {
                        $msg=Msg::new("\n\t\tlink-lowpri$n=$rhbn->{$key}{$sys} over UDP6\n\t\t  ip $address prefix $netmask port $port");
                    } else {
                        $msg=Msg::new("\n\t\tlink-lowpri$n=$rhbn->{$key}{$sys} over UDP\n\t\t  ip $address netmask $netmask port $port");
                    }
                    $lmsg.=$msg->{msg};
                } else {
                    $addrkey = "rdmalinklowpri${n}_address";
                    $maskkey = "rdmalinklowpri${n}_netmask";
                    $portkey = "rdmalinklowpri${n}_port";
                    my $rdmaudp = "rdmalinklowpri${n}_udp";
                    if (defined $rhbn->{$addrkey}) {
                        $address = $rhbn->{$addrkey}{$sys};
                        $netmask = $rhbn->{$maskkey}{$sys};
                        $port = $rhbn->{$portkey}{$sys};
                        my $rdmastr = defined $rhbn->{$rdmaudp} ? "UDP" : "RDMA";
                        $msg=Msg::new("\n\t\tlink-lowpri$n=$rhbn->{$key}{$sys} over $rdmastr\n\t\t  ip $address netmask $netmask port $port");
                        $lmsg.=$msg->{msg};
                    } else {
                        $msg=Msg::new("\n\t\tlink-lowpri$n=$rhbn->{$key}{$sys}");
                        $lmsg.=$msg->{msg};
                    }
                }
            }
            if ($prod->num_lopri_hbnics($rhbn) > 1) {
                $msg=Msg::new("\tLow-Priority Heartbeat NICs for $sys: $lmsg");
            } else {
                $msg=Msg::new("\tLow-Priority Heartbeat NIC for $sys: $lmsg");
            }
            $msg->print;
            $smsg .= "$msg->{msg}";
        }
    }

    if ($cfg->{lltoverudp} || $cfg->{lltovertcp}) {
        $msg = Msg::new("\nLLT tagging:\n");
        $msg->bold;
        $msg = Msg::new("When configuring LLT over UDP or TCP, tags are used in LLT configuration file with set-addr option.\nDue to this, LLT configuration file may use local NIC identifiers referring to remote IP addresses.\n");
        $msg->print;
    }

    # return the summary message
    $msg = Msg::new("Cluster configuration information:\n");
    $msg->{msg} .= $smsg;
    return $msg;
}


sub generate_udplinks_sys {
    my ($prod,$sys,$bcast) = @_;
    my ($sysi,$content,$plat,$n,$nic,$port,$system,$devn,$nid,$address,$linktype,$linktype_base,$dev,$bcastaddr,$vcs_link_str);
    my $cfg = Obj::cfg();

    $plat=EDRu::plat($prod->{padv});
    $sysi=$sys->{sys};
    if($plat =~ /AIX/m) {
        $dev = '/dev/xti/udp';
    } elsif ($plat =~ /SunOS|HPUX/m) {
        $dev = '/dev/udp';
    } else {
        $dev = 'udp';
    }
    $content = '';
    if($cfg->{lltovertcp}) {
        $linktype_base = 'tcp';
        $vcs_link_str = 'vcs_tcplink';
    } else {
        $linktype_base = 'udp';
        $vcs_link_str = 'vcs_udplink';
    }
    for my $n (1..$prod->num_hbnics_cfg($cfg)) {
        $nic = $cfg->{"vcs_lltlink$n"}{$sysi};
        $port = $cfg->{$vcs_link_str.$n.'_port'}{$sysi};
        $address = $cfg->{$vcs_link_str.$n.'_address'}{$sysi};
        $devn = $dev;
        $linktype = $linktype_base;
        if(EDRu::ip_is_ipv6($address)) {
            $devn = "$dev".'6';
            $linktype = $linktype.'6';
        }
        if($bcast){
            $bcastaddr=$sys->padv->nic_bcast_sys($sys,$nic,$address);
        }
        $bcastaddr="-" unless($bcastaddr);
        #$content .= "link $nic $devn - $linktype $port - $address -\n";
        if ($cfg->{lltovertcp}) {
            $content .= "link $nic $devn - $linktype $port - $address $bcastaddr\n";
            $content .= "set-dbg-minlinks 2\n";
        } else {
            $content .= "link $nic $devn - $linktype $port - $address $bcastaddr\n";
        }
    }
    for my $n (1..$prod->num_lopri_hbnics_cfg($cfg)) {
        $nic = $cfg->{"vcs_lltlinklowpri$n"}{$sysi};
        $port = $cfg->{"vcs_udplinklowpri$n".'_port'}{$sysi};
        $address = $cfg->{"vcs_udplinklowpri$n".'_address'}{$sysi};
        $devn = $dev;
        $linktype = $linktype_base;
        if(EDRu::ip_is_ipv6($address)) {
            $devn = "$dev".'6';
            $linktype = $linktype.'6';
        }
        if($bcast){
            $bcastaddr=$sys->padv->nic_bcast_sys($sys,$nic,$address);
        }
        $bcastaddr="-" unless($bcastaddr);
        #$content .= "link-lowpri $nic $devn - $linktype $port - $address -\n";
        $content .= "link-lowpri $nic $devn - $linktype $port - $address $bcastaddr\n";
    }
    $nid = 0;
    my $systems = cluster_systems();
    my $nodeid_hash = $prod->get_sys_nodeid_hash();
    for my $system (@$systems) {
        my $sysname;
        if(ref($system) eq 'Sys'){
            $sysname = transform_system_name($system->{sys});
        }else{
            $sysname = $system;
        }

        my $sysnamei = transform_system_name($sysi);
        if ($sysname eq $sysnamei) {
            next;
        }
        $nid = $nodeid_hash->{$sysname};
        for my $n (1..$prod->num_hbnics_cfg($cfg)) {
            $nic = $cfg->{"vcs_lltlink$n"}{$sysnamei};
            $address = $cfg->{$vcs_link_str.$n.'_address'}{$system->{sys}};
            $content.= "set-addr $nid $nic $address\n";
        }
        for my $n (1..$prod->num_lopri_hbnics_cfg($cfg)) {
            $nic = $cfg->{"vcs_lltlinklowpri$n"}{$sysnamei};
            $address = $cfg->{"vcs_udplinklowpri$n".'_address'}{$system->{sys}};
            $content.= "set-addr $nid $nic $address\n";
        }
    }
    $content.="set-bcasthb 0\n";
    $content.="set-arp 0\n";

    return $content;
}

#e3953503 -- CPI_7.3.1_P6.pl:line 608-902 -- The "-version" option of InfoScale 7.3.1 installer fails to download the available Maintenance or Patch releases
package Rel;
use strict;

sub vc_download_patches {
    my ($rel) = @_;
    my ($cpic, $msg, $ayn, $padv, $sysname, $download_path, $menuopt, $choice, $backopt);
    my ($fix, $relname, @items, $filename, $target_dir, $localsys);
    my ($flag, $download_errors, $errmsg);
    my (@available_patch, @available_hotfix);
    my ($no_release, $ident_systems, $deploy);

    #require Deploy;
    #if(Obj::created("Deploy")){
    #    $deploy=Obj::deploy();
    #} else {
    #    $deploy=Deploy->new();
    #    $deploy->init_preference_hash();
    #}
    #$deploy->read_deploy_config();

    if ($rel->{sort_ready_for_vc} == 0) {
        $msg = Msg::new("Version checker cannot download updates now since the SORT website is not available.", 40, 6220);
        $msg->print;
        return '';
    }

    $flag = 0;
    for my $sysname (@{$rel->{vc_sys}}) {
        if (@{$rel->{vc_update_status}{$sysname}{available_patch_update}{id_set}} != 0 or @{$rel->{vc_update_status}{$sysname}{available_hotfix_update}{id_set}} != 0) {
            $flag = 1;
        }
    }
    if ($flag == 0) {
        $msg = Msg::new("No available Maintenance or Patch releases to be downloaded.", 40, 6429);
        $msg->print;
        return '';
    }
    $cpic = Obj::cpic();
    $backopt = 0;
    Msg::n();
    $msg = Msg::new("Would you like to download the available Maintenance or Patch releases that cannot be found in the repository?", 40, 6430);
    $ayn = $msg->aynn();
    Msg::n();
    if ($ayn eq 'Y') {
	# Below code will set repository path for downloading suggested releases.
        my $repo_dir = $rel->repository_set_home();
        $rel->{repository_path_for_vc} = $repo_dir;
        $rel->{vc_download_path} = $rel->{repository_path_for_vc};
        $rel->{repos_set_for_vc} = 1;
        Msg::n();
        $msg = Msg::new("Base releases must be manually downloaded from FileConnect and loaded using the R - Manage Repository Images option from the Deployment Server.", 40, 6663);
        $msg->print;
        Msg::n();
        $msg = Msg::new("Public Maintenance and Patch releases will be downloaded into $rel->{vc_download_path} on the local system", 40, 6431, "$rel->{vc_download_path}");
        $msg->print;

        $localsys=EDR::get('localsys');

        $download_path = $rel->{vc_download_path};
        chomp($download_path);
        $localsys->mkdir($download_path);

        for my $prodinfo (keys %{$rel->{prodsys}}) {
            my $sysname = $rel->{prodsys}{$prodinfo}[0];
            $ident_systems= join(' ', @{$rel->{prodsys}{$prodinfo}});
            while(1) {
                last if (!$rel->get_download_releases($sysname, $ident_systems, \@available_patch, \@available_hotfix));
                my (@download_ids, $skip, $url);
                my ($msg_option_maintenance, $msg_option_hotfix, $msg_skip, @available_updates_for_menu, $required_size, $successful_msg, $expand_file,$release_name_in_dir, $expand_file_without_gz, $release_dir, $cksum_correctmsg, $cksum_errmsg, $right_cksum, $dir, $releasetype, $uploadfilename, $result, $cksum_result);
                $skip = 0;
                $padv = $rel->{vc_host_padv}{$sysname};
                $no_release = 0;

                $msg = Msg::new("For $ident_systems, please select the level of release you want to download:", 40, 6226, "$ident_systems");
                $msg_option_maintenance = Msg::new("Maintenance", 40, 6227);
                $msg_option_hotfix = Msg::new("Patch", 40, 6432);
                $msg_skip = Msg::new("Skip this step", 40, 6229);

                push @available_updates_for_menu, $msg_option_maintenance->{msg};
                push @available_updates_for_menu, $msg_option_hotfix->{msg};
                push @available_updates_for_menu, $msg_skip->{msg};
                $menuopt = EDRu::arruniq(@available_updates_for_menu);
                $choice = $msg->menu($menuopt, '', '', $backopt, 0);
                Msg::n();
                $fix = $available_updates_for_menu[$choice-1];
                if ($fix eq "Maintenance") {
                    if (@available_patch) {
                        if (!$rel->select_releases($sysname, $ident_systems, 'maintenance', \@download_ids, @available_patch)) {
                            next;
                        }
                    } else {
                        $msg = Msg::new("For $ident_systems, there are no available Maintenance releases to be downloaded.", 40, 6230, "$ident_systems");
                        $msg->print;
                        Msg::n();
                        $no_release = 1;
                    }
                }
                if ($fix eq "Patch") {
                    if (@available_hotfix) {
                        if (!$rel->select_releases($sysname, $ident_systems, 'patch', \@download_ids, @available_hotfix)) {
                            next;
                        }
                    } else {
                        $msg = Msg::new("For $ident_systems, there are no available Patch releases to be downloaded.", 40, 6433, "$ident_systems");
                        $msg->print;
                        Msg::n();
                        $no_release = 1;
                    }
                }

                if ($fix eq "Skip this step") {
                    $skip = 1;
                    last;
                }

                if ($no_release== 1 || !@download_ids) {
                    $msg = Msg::new("Would you like to download another available update release for $ident_systems?", 40, 6232, "$ident_systems");
                    $ayn = $msg->aynn();
                    Msg::n();
                    if ($ayn eq 'Y') {
                        next;
                    } else {
                        last;
                    }
                }
                $required_size = 0;
                for my $id (@download_ids) {
                    $required_size += $rel->{vc_update_status}{$sysname}{available_update}{id_detail}{$id}{upload_size};
                }
                $required_size = $required_size / 1024 / 1024;
                $required_size = (int($required_size) == $required_size) ? $required_size : (int($required_size) + 1);
                my $dfk=$localsys->cmd("_cmd_dfk $download_path 2>/dev/null");
                my @dfk=split(/\n/,$dfk);
                $dfk=pop @dfk;
                @dfk=split(/\s+/m,$dfk);
                my $dfm = int($dfk[3] / 1024);
                if ( $dfm < $required_size ) {
                    $msg=Msg::new("$required_size MB is required for your selections in the $download_path directory and only $dfm MB is available on the local system", 40, 6492, "$required_size", "$download_path", "$dfm");
                    $msg->error();
                    return 1;
                }
                Msg::n();

                $dir = "$download_path/"."patch/"."targz/";
                $localsys->mkdir($dir);
                $dir = "$download_path/"."hotfix/"."targz/";
                $localsys->mkdir($dir);

                foreach my $id (@download_ids) {
                    $relname = $rel->{vc_update_status}{$sysname}{available_update}{id_detail}{$id}{release_name};
                    $url = $rel->{vc_update_status}{$sysname}{available_update}{id_detail}{$id}{upload_location};
                    $releasetype = $rel->{vc_update_status}{$sysname}{available_update}{id_detail}{$id}{update_type};

                    $target_dir = "$download_path/"."$releasetype/"."targz/";

                    @items = split(/\//, $url);
                    $uploadfilename = $items[-1];

                    $filename = "$target_dir"."$uploadfilename";
                    if (-f $filename) {
                        $result = $localsys->cmd("_cmd_cksum $filename");
                        @items = split(/\s+/, $result);
                        $cksum_result = $items[0];
                        $right_cksum = $rel->{vc_update_status}{$sysname}{available_update}{id_detail}{$id}{upload_cksum};
                        if ($cksum_result ne $right_cksum) {
                            $cksum_errmsg=Msg::new("Download the existing file $filename again since the ckeck sum is different\n", 40, 6234, "$filename");
                            $cksum_errmsg->log;
                            $cksum_errmsg->print;
                        } else {
                            $cksum_correctmsg=Msg::new("$relname is already under $target_dir\n", 40, 6235, "$relname", "$target_dir");
                            $cksum_correctmsg->log;
                            $cksum_correctmsg->print;
                            $rel->{vc_update_status}{$sysname}{available_update}{id_detail}{$id}{in_repository} = 1;
                            next
                        }
                    }

                    $release_name_in_dir = $uploadfilename;
                    $release_name_in_dir =~ s/\.tar\.gz$//g;

                    $msg=Msg::new("$relname:", 40, 6236, "$relname");
                    $msg->print;
                    my $ret = EDRu::download_file($url, $filename, 1, 20);
                    if (!$ret) {
                        #Error occurred
                        $download_errors=EDR::get('download_errors');
                        $errmsg=Msg::new("Could not download $relname!\n$download_errors\n", 40, 6237, "$relname", "$download_errors");
                        $errmsg->log;
                        $errmsg->print;
                    }
                    $result = $localsys->cmd("_cmd_cksum $filename");

                    @items = split(/\s+/, $result);
                    $cksum_result = $items[0];
                    $right_cksum = $rel->{vc_update_status}{$sysname}{available_update}{id_detail}{$id}{upload_cksum};
                    if ($cksum_result ne $right_cksum) {
                        $cksum_errmsg=Msg::new("Download $relname unsuccessfully!\nCheck sum is different\n", 40, 6238, "$relname");
                        $cksum_errmsg->log;
                        $cksum_errmsg->print;
                    } else {
                        $release_dir = "$download_path/"."$releasetype/"."images/"."$release_name_in_dir/";
                        $localsys->mkdir($release_dir);
                        $localsys->copyfile($filename, $release_dir);
                        $expand_file = "$release_dir"."$uploadfilename";
                        $localsys->cmd("_cmd_gunzip $expand_file") if(-f $expand_file);
                        $expand_file_without_gz = $expand_file;
                        $expand_file_without_gz =~ s/\.gz$//g;
                        $localsys->cmd("cd $release_dir; _cmd_tar -xvf $expand_file_without_gz");
                        $localsys->cmd("_cmd_rmr $expand_file_without_gz");

                        $successful_msg = "Download $relname successfully!\n  Location: $target_dir\n  Expanded files Location: $release_dir";
                        $cksum_correctmsg=Msg::new("$successful_msg\n", 40, 6239, "$successful_msg");
                        $cksum_correctmsg->log;
                        $cksum_correctmsg->print;
                        $rel->{vc_update_status}{$sysname}{available_update}{id_detail}{$id}{in_repository} = 1;
                    }
                    Msg::n();
                }
                $msg = Msg::new("Do you want to download another available Maintenance or Patch release for $ident_systems?", 40, 6434, "$ident_systems");
                $ayn = $msg->aynn();
                Msg::n();
                if ($ayn eq 'Y') {
                    next;
                } else {
                    last;
                }
            }
        }
    }
    return '';
}

# This function set repository path to download suggested releases.
sub repository_set_home {
    my ($rel) = @_;
    my $default_repo_path = '/opt/VRTS/repository';
    my ($msg,$dir,$localsys,$result,$repository);
    my $cpic=Obj::cpic();
    my $edr=Obj::edr();
    while (1) {
        Msg::n();
        $msg=Msg::new("In which directory would you like to store release images? :", 60, 1022);
        $dir=$msg->ask($default_repo_path);
        if ($dir !~ m{^/}) {
            $dir=EDRu::abs_path($dir);
            $msg=Msg::new("Would you like to create repository directory under $dir?", 60, 1023, "$dir");
            next if ($msg->ayny eq 'N');
        }
        $dir=$cpic->path($dir);
        $localsys=$edr->{localsys};
        last if ($rel->verify_repository($localsys,$dir));
    }
    return $dir;
}

# Perform the following tasks:
# 1. Verify dir
# 2. Create dir
# 3. Change dir permission to 755
sub verify_repository {
    my ($rel,$localsys,$dir) = @_;
    my ($result,$msg);
    my ($files,@files);
    my $sub_dirs = ["patch","hotfix","ga",".cpihotfix",".matrix",".lock"];
    if ($localsys->is_dir($dir)) {
        $msg=Msg::new("The directory $dir already exists on $localsys->{sys}", 60, 1115, "$dir", "$localsys->{sys}");
        $msg->log;
        $files=$localsys->lsfile($dir);
        @files=split(/\n/,$files);
        $files=EDRu::arrdel(\@files, @{$sub_dirs});
        if (@$files > 0) {
            # There're irrelevant files in the repository dir
            # print warning message.
            $msg=Msg::new("The directory $dir already exists and contains non-repository files on $localsys->{sys}.  It is recommended to create a new repository directory that does not contain any unrelated files.", 60, 1116, "$dir", "$localsys->{sys}");
            $msg->warning;
            $msg=Msg::new("Do you want to continue?", 60, 1117);
            if ($msg->aynn() eq 'Y') {
                # User still want to use this dir
                $localsys->chmod($dir,'755');
                return 1;
            } else {
                return 0;
            }
        }
    } else {
        $result=$localsys->mkdir($dir) if (!-d $dir);
        if (!$localsys->is_dir($dir)) {
            $msg=Msg::new("Cannot create $dir directory on $localsys->{sys}", 60, 1118, "$dir", "$localsys->{sys}");
            $msg->error();
            return 0;
        }
    }
    $localsys->chmod($dir,'755');
    return 1;
}

#e3957571 -- CPI_7.3.1_P7.pl 956-1040: Infoscale 7.3.1 installer -stop operation skip stopping veki process
package Rel;
use strict;
use LWP::UserAgent;
use JSON;
@Rel::ISA = qw(CPIP);

sub adjust_procs_list {}

package Rel::UXRT731::RHEL7x8664;
@Rel::UXRT731::RHEL7x8664::ISA = qw(Rel::UXRT731::Linux);

# veki found loaded after 'installer -stop' operation hence considering veki for installer stop operation
sub adjust_procs_list {
    my ($rel, $cpic) = @_;
    if ( !EDRu::inarr('veki731', @{$cpic->{stopprocs}})){
        push (@{$cpic->{stopprocs}}, 'veki731');
    }
    foreach my $sys (@{$cpic->{systems}}) {
        if(!EDRu::inarr('veki731', @{$sys->{stopprocs}})) {
            push (@{$sys->{stopprocs}}, 'veki731');
        }
    }
    return;
}

package Prod::FS731::Common;
@Prod::FS731::Common::ISA = qw(Prod);

sub verify_procs_list {
    my ($prod,$procs,$state)=@_;
    my ($vm,$cpic,$rel,$cprod,$rcprod,$previouscomponent,$activecomponent,$rtn);

    $cpic = Obj::cpic();
    $rel = $cpic->rel;
    $cprod=CPIC::get('prod');
    $rcprod=$rel->prod($cprod);
    $previouscomponent=${$rcprod->{previouscomponent}}[0] if ($rcprod->{previouscomponent});
    $activecomponent = $rel->get_activecomponent();
    $rtn = $rel->{reconfig_matrix}{$previouscomponent}{$activecomponent} if ($previouscomponent && $activecomponent);

    if ((defined $state && $state eq 'stop') && (EDRu::inarr('vxsvc34', @{$procs}))) {
        $procs=EDRu::arrdel($procs, 'vxsvc34');
        unshift(@{$procs}, 'vxsvc34');
    }
    if ((defined $state && $state eq 'stop') && (Cfg::opt('configure')) && ($rtn !~ /(AR|SUS)$/)) {
        $procs = $prod->remove_procs_for_prod($procs);
    }
    # Below code sets vxglm stop for SFHA and SF stop/configure operations. System specific stop will be decided later.
    if ($activecomponent =~ /^(SFHA731|SF731)$/ && (defined $state && $state eq 'stop') && ( Cfg::opt('stop') || Cfg::opt('configure') ) && !Cfg::opt('install')) {
        my $proc_name = 'vxglm731';
        if (!EDRu::inarr($proc_name, @{$procs})){
            unshift(@{$procs}, $proc_name);
        }
    }
    return $procs;
}

sub verify_procs_list_sys {
    my ($prod,$sys,$procs,$state)=@_;
    my ($vm,$cpic,$rel,$cprod,$rcprod,$previouscomponent,$activecomponent,$rtn);

    $cpic = Obj::cpic();
    $rel = $cpic->rel;
    $cprod=CPIC::get('prod');
    $rcprod=$rel->prod($cprod);
    $vm = $sys->prod('VM731');
    # adjust vxsvc process
    $procs = $vm->adjust_vxsvc_for_procs_sys($sys,$procs,$state);
    # adjust sfmh-discovery process
    $procs = $vm->adjust_sfmh_for_procs_sys($sys,$procs,$state);

    $previouscomponent=${$rcprod->{previouscomponent}}[0] if ($rcprod->{previouscomponent});
    $activecomponent = $rel->get_activecomponent();
    $rtn = $rel->{reconfig_matrix}{$previouscomponent}{$activecomponent} if ($previouscomponent && $activecomponent);

    if ((defined $state && $state eq 'stop') && (Cfg::opt('configure')) && ($rtn !~ /(AR|SUS)$/)) {
        $procs = $prod->remove_procs_for_prod($procs);
    }
    # Below code sets vxglm unload for system, If found running during SFHA and SF stop/configure operations.
    if ($activecomponent =~ /^(SFHA731|SF731)$/ && (defined $state && $state eq 'stop') && ( Cfg::opt('stop') || Cfg::opt('configure') ) && !Cfg::opt('install')) {
        my $proc_name = 'vxglm731';
        my $proc_obj=$sys->proc($proc_name);
        if (!EDRu::inarr($proc_name, @{$procs}) && $proc_obj->check_sys($sys)){
            unshift(@{$procs}, $proc_name);
        }
    }
    return $procs;
}

1;
